module spt

  use settings
  use MatrixUtils
  use cmbtypes
  use foregrounds
  
  implicit none
  integer, parameter :: llmax = 10000

    ! The bandpowers, in order 150, 150x220, 220
    real, dimension(1:15,3) :: spec

    ! The cholesky factored bandpower covariance
    real, dimension(1:45,1:45) :: cov

    ! The window functions
    real, dimension(50:llmax,1:15) :: windows

    real, dimension(0:llmax,1:3,1:2) :: db_err

    ! Effective frequencies for the bands
    ! The order is (dutsy clustered, dusty poisson, radio poisson, ksz, tsz)
    real, dimension(5,2), parameter :: spt_eff_fr \
        = (/ (/ 154.2, 154.2, 151.7, 150, 153/), \
             (/ 221.4, 221.4, 219.3, 220, 220.2/) /)
!used band_centers2 & band_centers_sz
!alph=-0.41 for radio and  3.79 for dusty

    ! Everything is normalized to the power in the 150 spectrum
    real, dimension(5), parameter :: spt_norm_fr = spt_eff_fr(:,1)
    
    real,dimension(2:llmax) :: cls_diff

contains
  subroutine InitLensingTemplate(LensingTemplate)
    integer :: ll,ii
    character(len=120) :: file
    character*(*) :: LensingTemplate
    
    if (LensingTemplate/='') then
       open(13,file=LensingTemplate)
       do ll=2,llmax
          read(13,*) ii, cls_diff(ii)
          cls_diff(ii)=cls_diff(ii)/((ii+1.)*ii)*twopi
       enddo
       close(13)
    else
       cls_diff(:)=0.0
    end if

  end subroutine InitLensingTemplate

    ! Load the bandpowers, covariance, and window functions
  subroutine InitSptData(bp_file,cov_file,window_folder,f_berr_stub)

        character(*) :: bp_file, cov_file, window_folder,f_berr_stub
        integer i,j,k,dum

        ! Read in bandpowers
        call OpenTxtFile(bp_file, tmp_file_unit)
        do i=1,3
            do j=1,15
                read (tmp_file_unit,*) dum,spec(j,i)
            end do
        end do
        close(tmp_file_unit)


        ! Read in covariance
        call OpenTxtFile(cov_file, tmp_file_unit)
        do i=1,45
            do j=1,45
                read (tmp_file_unit,*) cov(i,j)
            end do
        end do
        close (tmp_file_unit)
        call Matrix_Cholesky(cov)


        ! Read in windows
        do i=1,15
            call OpenTxtFile(window_folder//trim(numcat('window_',i)),tmp_file_unit)
            do j=50,llmax
                read (tmp_file_unit,*) dum, windows(j,i)
            end do
            close(tmp_file_unit)
        end do

        call OpenTxtFile(f_berr_stub//'150.txt',tmp_file_unit)
        do j=0,llmax
           read (tmp_file_unit,*) dum, db_err(j,1,1), db_err(j,2,1), db_err(j,3,1)
        end do
        call OpenTxtFile(f_berr_stub//'220.txt',tmp_file_unit)
        do j=0,llmax
           read (tmp_file_unit,*) dum, db_err(j,1,2), db_err(j,2,2), db_err(j,3,2)
        end do
        close(tmp_file_unit)
        

    end subroutine


    function SptLnLike(cmbcls, freq_params)

        real, dimension(:,:), intent(in) :: cmbcls
        real, dimension(:) :: freq_params
        real :: SptLnLike

        real, dimension(1:15,3) :: cb
        real, dimension(3,3) :: cl_arr
        real, dimension(3) :: cl
        real, dimension(5) :: dum
        real, dimension(1:45) :: deltacb
        real, dimension(2) :: a_calib
        real, dimension(3,2) :: dberrfac
        real :: cmbcl
        integer l,b
	type(foreground_params) :: foregrounds

        a_calib = freq_params(13:14)
        dberrfac(:,1)=freq_params(15:17)
        dberrfac(:,2)=freq_params(18:20)

	foregrounds = GetForegroundParamsFromArray(freq_params(1:12))


        do b=1,15
            cb(b,:) = (/ 0, 0, 0 /)
        end do

!        write(*,*)'in spt code'
!        open (33,file='suxp_lma10000_spt')
        do l=1500,llmax

            ! Get the CMB TT power at the current l

           ! extrapolation already done now
           cmbcl = cmbcls(l,1)
!           cmbcl = extrapolateCmb(cmbcls(:,1),l)
!            write(33,'(I,6G)')l,cmbcl

            
            ! Add the frequency dependent components to CMB and adjust beam

            cl(1) = (cl_foreground(foregrounds,l,1,1,2,spt_eff_fr,spt_norm_fr) + cmbcl) * beam_adj(dberrfac,1,1,l)
            cl(2) = (cl_foreground(foregrounds,l,1,2,2,spt_eff_fr,spt_norm_fr) + cmbcl) * beam_adj(dberrfac,1,2,l)
            cl(3) = (cl_foreground(foregrounds,l,2,2,2,spt_eff_fr,spt_norm_fr) + cmbcl) * beam_adj(dberrfac,2,2,l)

            cl(:) = cl(:)*l*(l+1)/(2*pi)

            ! Distribute this c_l to the bands its windowed in
            do b=1,15
                cb(b,:) = cb(b,:) + windows(l,b)*cl(:)
            end do

 !           write(33,'(I,6G,6G,6G,6G,6G,6G,6G)')l,cl(1)*a_calib(1) * a_calib(1),&
 !                cl(2)*a_calib(1) * a_calib(2),cl(3)*a_calib(2) * a_calib(2),&
 !                cmbcl,&
 !                cl_foreground(foregrounds,l,1,1,2,spt_eff_fr,spt_norm_fr),&
 !                cl_foreground(foregrounds,l,1,2,2,spt_eff_fr,spt_norm_fr),&
 !                cl_foreground(foregrounds,l,2,2,2,spt_eff_fr,spt_norm_fr)
            
        end do
 !       close(33)
        ! Apply calibration parameters
        cb(:,1) = a_calib(1) * a_calib(1) * cb(:,1)
        cb(:,2) = a_calib(1) * a_calib(2) * cb(:,2)
        cb(:,3) = a_calib(2) * a_calib(2) * cb(:,3)

        ! Stack the bandpowers, indexed in the same way as our bandpower covariance
        deltacb(1:15)  = cb(:,1) - spec(:,1)
        deltacb(16:30) = cb(:,2) - spec(:,2)
        deltacb(31:45) = cb(:,3) - spec(:,3)

        SptLnLike = Matrix_GaussianLogLikeChol(cov,deltacb) 
	SptLnLike = SptLnLike + & 
      SptLnPrior(a_calib(1),a_calib(2),dberrfac,foregrounds)

    end function


    function SptLnPrior(a150,a220,dberrfac,foregrounds)

        real :: a150, a220, SptLnPrior
        real, dimension(3,2) :: dberrfac
	type(foreground_params) :: foregrounds
        real :: del_alpha
        real :: cirrus_factor
        integer i,j
        real fradio, radio_amp,radio_unc
        
        ! calibration prior
        
        SptLnPrior = (a150-1)**2/(2*.0352**2) + (a220/a150 - 1)**2 / (2*.062**2)

        ! pre v3 beams = dropped Oct 1, 2010
!        SptLnPrior = (a150-1)**2/(2*.036**2) + (a220/a150 - 1)**2 / (2*.062**2)
        ! beam prior
        !changed to 2.86 from 2.85 on 8/10/10 CR
        !SptLnPrior = SptLnPrior + dsigmab150**2/(2*(2.86e-6)**2) + dsigmab220**2/(2*(6.83e-6)**2)
        ! changed 9/20/10 for new beams
        !each parameter has unit variance
        do i=1,2 
           do j=1,3 
              SptLnPrior = SptLnPrior + dberrfac(j,i)**2 / 2
           end do
        enddo

        del_alpha = GetAlphaPrior()
        ! dusty index prior
        if ( del_alpha > 0) then
           SptLnPrior = SptLnPrior + (foregrounds%alpha_dg_po-foregrounds%alpha_dg_cl)**2/(2*(del_alpha)**2)
        end if

        !cirrus prior
        cirrus_factor = GetCirrusFactor()
        if (cirrus_factor > 0) then 
           SptLnPrior = SptLnPrior + (foregrounds%czero_cirrus(1) - 0.605*cirrus_factor)**2 / (2*(0.194)**2)
           SptLnPrior = SptLnPrior + (foregrounds%czero_cirrus(2) - 3.673*cirrus_factor)**2 / (2*(0.845)**2)
           
        end if

        radio_amp = GetRadioAmpPrior()
        radio_unc = GetRadioAmpUncPrior()
        if (radio_amp >= 0 .and. radio_unc > 0) then
           fradio =(foregrounds%czero_rg - radio_amp) / radio_unc
           SptLnPrior = SptLnPrior + fradio**2 / 2
           if (foregrounds%czero_rg < 0) then
              SptLnPrior = SptLnPrior + 1e6
           endif
        endif
    end function


    !
    ! Gets the beam spectrum at l, given the main lobe beam width
    !
    function beam_l(sigma_b,l)

        real, parameter :: a=.85

        real :: sigma_b
        real beam_l
        integer l

        beam_l = (1.-a) * exp(-.5 * (.00292*l)**1.8) + a * exp(-.5 * (sigma_b*l)**1.5)

    end function


    !
    ! The factor we need to multiply a given C_l by to convert it
    ! from the nominal beam shape to the one described by dsigma_b
    !
    function beam_adj(errfac,fri,frj,l)

!        real, dimension(2) :: dsigma_b, sigma_b
!        real, dimension(2), parameter :: sigma_b0 = (/ 0.00017172336, 0.00016751516 /)
!        integer :: fri, frj, l
!        real :: beam_adj

!        sigma_b = sigma_b0 + dsigma_b

!        beam_adj = beam_l(sigma_b0(fri),l)*beam_l(sigma_b0(frj),l)/beam_l(sigma_b(fri),l)/beam_l(sigma_b(frj),l)

!      real, dimension(2) :: inner,dc
      real, dimension(3,2) :: errfac
      integer :: fri, frj, l,j
      real :: beam_adj
      beam_adj=1
      do j=1,3
         beam_adj = beam_adj* (1 + errfac(j,fri)*db_err(l,j,fri))* &
              (1 + errfac(j,frj)*db_err(l,j,frj))
      end do


    end function beam_adj
    

    ! cmbcl is the cmb power spectrum
    ! if l is < ubound(cmbcl) returns cmbcl(l) otherwise
    ! returns the extrapolated power spectrum at l, taking a powerlaw above 3000
    function extrapolateCmb(cmbcl,l)

        integer :: l, l1, l2
        real, dimension(:) :: cmbcl
        real :: extrapolateCmb
        integer ll,ii
        real :: cleff1, cleff2
        character(LEN=Ini_max_string_len) LenseTemplateFile
           

        l1 = 3000
        l2 = ubound(cmbcl,dim=1)
        
        if (l<l2) then
            extrapolateCmb = cmbcl(l)
        else
           cleff1 = cmbcl(l1)-cls_diff(l1)
           cleff2 = cmbcl(l2)-cls_diff(l2)
           extrapolateCmb = cleff2 * (real(l)/l2) ** ( log(cleff2/cleff1) / log(real(l2)/l1) ) 
           if (l <= llmax) then
              extrapolateCmb = extrapolateCmb +cls_diff(l)
           end if
        end if
        
    end function

end module spt
